<?php

/**
 * @file
 * Handles importing of terms.
 */

/**
 * This is a "data callback" function that is called by the import for
 * each data object loaded from an import file. This function is responsible
 * for calling the "process callback" which does the actual updating/importing
 * tasks.
 *
 * @param $data
 * @param $process
 * @param &$context
 * @return void
 */
function mover_import_terms_process_data($data, $process, &$context) {

  // Get information about the process
  $process_info = mover_import_get_process($process);

  // Always start a new object for term data. It's not reliable to
  // trust the mover map for terms. Terms are different than nodes
  // in that there should never be more than one term in at the same place
  // in the hierarchy of a given vocabulary.
  $term = mover_term_clone($data);

  // Let the process callback function handle the real work
  $function = $process_info['process callback'];
  if (function_exists($function)) {
    return $function($term, $data);
  }
  else {
    drupal_set_message(t('Function %function not found!', array(
      '%function' => $function)), 'error');
  }
}

/**
 * Creates or returns a new term based on raw term data.
 */
function mover_import_terms_process_import(&$term, $data) {

  // Figure out which vocabulary to map this term to. There should
  // be an implementation of the "map_vocab" hook that sets
  // $term->vid.
  mover_import_invoke('map_vocab', $term, $data);

  // If the vocabulary couldn't be determined then this term can't be imported.
  if (empty($term->vid)) {
    watchdog('mover_import', 'Vocabulary not found for term @name (VID @vid)', array(
      '@name' => $data->name, '@vid' => $data->vid), WATCHDOG_NOTICE);
    return FALSE;
  }

  // "Term path" is a unique array generated by Mover Export, and that will be
  // used here to import the term hierarchy. Otherwise the term hierarchy might
  // be lost, e.g. About/Team might be imported as just "Team". By exporting
  // this data as array('About', 'Team') we can import and rebuild the hierarchy
  // as it was on the old site.
  //
  // Ensure term path is an array
  if (!is_array($data->path)) {
    $data->path = array($data->name);
  }

  // Allow another module to alter the importing of this term
  mover_import_invoke('import_term', $term, $data);

  // This function find or creates new terms in a way that doesn't import
  // duplicates. There should only be one unique term
  // name at a given position in the hierarchy of a vocabulary.
  $term->tid = mover_import_get_or_create_tid($term->vid, $data);

  // Load the term that was just saved
  //$term = taxonomy_get_term($term->tid);

  // Update other attributes of the term, just in case the description or anything
  // else needs to be updated.
  //$form_values = (array)$term;
  //taxonomy_save_term($form_values);

  // If we have TID
  if ($term->tid) {

    // Create URL redirects for old URL aliases
    if (is_array($data->aliases)) {
      foreach ($data->aliases as $url_alias) {
        mover_import_add_path_redirect($url_alias, 'taxonomy/term/'. $term->tid);
      }
    }

    // Create URL redirects for old path redirects
    if (is_array($data->redirects)) {
      foreach ($data->redirects as $redirect) {
        mover_import_add_path_redirect($redirect, 'taxonomy/term/'. $term->tid);
      }
    }

    // Let other modules do other things with the finished term mapping
    mover_import_invoke('import_term_postprocess', $term, $data);

    // Save this mapping for later
    $map_id = db_result(db_query("SELECT map_id FROM {mover_map} WHERE type='term' AND new_content_id=%d AND old_content_id=%d", $term->tid, $data->tid));
    if ($map_id) {
      db_query("UPDATE {mover_map} SET updated=%d WHERE map_id=%d", time(), $term->tid);
    }
    else {
      // Terms can't map multiple ways, or at least that's not currently supported by the Mover module
      db_query("DELETE FROM {mover_map} WHERE type='term' AND (new_content_id=%d OR old_content_id=%d)", $term->tid, $data->tid);
      db_query("INSERT INTO {mover_map} (type, new_content_id, old_content_id, imported) VALUES ('term', %d, %d, %d)", $term->tid, $data->tid, time());
    }

    return $term;
  }
}

/**
 * Manually load an old term record from the import directory.
 *
 * @param $old_tid
 * @return $term object
 */
function mover_import_load_old_term($old_tid) {
  return mover_import_load_old('terms', $old_tid);
}

/**
 * Prepare a clone of a term during import.
 */
function mover_term_clone($original_term) {
  $term = drupal_clone($original_term);
  unset($term->tid);
  return $term;
}

/**
 * Gets the VID from a vocabulary name, with an optional
 * parameter for finding non-exact matches.
 *
 * @param string $vocab_name
 * @param int $similarity
 *   If set to less than 100 then a vocabulary may be
 *   returned with a name that is not 100% identical to the
 *   name requested.
 * @return int $vid
 */
function mover_import_get_vid($vocab_name, $similarity = 100) {
  $vocabularies = taxonomy_get_vocabularies();

  $matches = array();
  foreach ($vocabularies as $vocab) {
    // If the vocab name matches an existing vocab, that's likely it!
    if ($vocab->name == $vocab_name) {
      $matches[100] = $vocab->vid;
      break;
    }
    else {
      $percent = 0;
      similar_text($vocab->name, $vocab_name, $percent);
      $matches[$percent] = $vocab->vid;
    }
  }

  // Sort the matches by similarity
  krsort($matches);

  // Return the result, if there's a close enough match
  $vid = 0;
  $best_percent = array_shift(array_keys($matches));
  if ($best_percent >= $similarity) {
    $vid = array_shift($matches);
  }
  return $vid;
}

/**
 * A simple function for finding or creating a term by name within a specified vocab.
 *
 * The $data param should contain one or more of these ways of finding a term:
 *  $data['path'] (see below)
 *  $data['tid'] (the old TID, useful for saving the old <-> new term relationship
 *
 * The term_path is an array of parent term names followed by the
 * term name you are looking for. For example, let's say you have a
 * vocubulary for organizing food. There is a parent term Fruit with
 * subterms Apples and Pears:
 * - Fruit
 *   - Apples
 *   - Pears
 * If you want to find the TID for Pears, you would do this:
 * $tid = mover_get_or_create_tid($vid, array('path' => array('Fruit', 'Pears')));
 *
 * When the optional $create boolean value is TRUE (as it is by default),
 * you can also create a new TID for Oranges:
 * $tid = mover_get_or_create_tid($vid, array('path' => array('Fruit', 'Oranges')));
 *
 * Or you can even go directly to a subterm of a term that doesn't yet
 * exist and this function will create both (and return the last):
 * $tid = mover_get_or_create_tid($vid, array('path' => array('Fruit', 'Grapefruit', 'Ruby Red Grapefruit')));
 */
function mover_import_get_or_create_tid($vid, $data, $create = TRUE) {
  if (is_object($data)) {
    $data = (array)$data;
  }
  $term_path = $data['path'];

  $parent_tid = 0;
  $weight = 0;
  $description = (empty($data['description'])) ? '' : $data['description'];
  $tid = 0;
  if (!is_array($term_path)) {
    $term_path = array($term_path);
  }
  foreach ($term_path as $name) {
    $name = trim($name);

    // Try to find an existing term
    $tid = db_result(db_query("SELECT td.tid FROM {term_data} td
      INNER JOIN {term_hierarchy} th ON td.tid=th.tid
      WHERE td.vid = %d AND th.parent = %d AND td.name = '%s' LIMIT 1", $vid, $parent_tid, $name));

    $values = array('vid' => $vid, 'name' => $name, 'description' => $description, 'weight' => $weight);
    if ($tid) {
      $values['tid'] = $tid;
      // Not much to do with an existing term since the name, vid, and hierarchy
      // is correct because of how the term is found (above). We can update the
      // description and weight though.
      drupal_write_record('term_data', $values, 'tid');
    }
    // Add the term
    elseif ($create) {
      drupal_write_record('term_data', $values);
      $tid = $values['tid'];
      db_query("INSERT INTO {term_hierarchy} (tid, parent) VALUES (%d, %d)", $tid, $parent_tid);
    }
    $parent_tid = $tid;
    $weight++;
  }
  return $tid;
}

/**
 * Implementation of hook_taxonomy().
 */
function mover_import_taxonomy($op, $type, $edit) {
  if ($type == 'term' && ($op == 'delete')) {
    db_query("DELETE FROM {mover_map} WHERE type='term' AND new_content_id=%d", $edit['tid']);
  }
}
